"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.assertState = exports.deleteState = void 0;
const client_s3_1 = require("@aws-sdk/client-s3");
const client_dynamodb_1 = require("@aws-sdk/client-dynamodb");
const utils_log_1 = require("@goldstack/utils-log");
const assertDynamoDBTable = async (params) => {
    // defining a table as required by Terraform https://www.terraform.io/docs/language/settings/backends/s3.html#dynamodb_table
    try {
        await params.dynamoDB.send(new client_dynamodb_1.CreateTableCommand({
            AttributeDefinitions: [
                {
                    AttributeName: 'LockID',
                    AttributeType: 'S',
                },
            ],
            KeySchema: [
                {
                    AttributeName: 'LockID',
                    KeyType: 'HASH',
                },
            ],
            TableName: params.tableName,
            BillingMode: 'PAY_PER_REQUEST',
        }));
        (0, utils_log_1.info)('DynamoDB table created for terraform state', {
            tableName: params.tableName,
        });
    }
    catch (e) {
        if (!(e instanceof client_dynamodb_1.ResourceInUseException)) {
            throw new Error(e.message);
        }
    }
};
const deleteDynamoDBTable = async (params) => {
    try {
        await params.dynamoDB.send(new client_dynamodb_1.DeleteTableCommand({
            TableName: params.tableName,
        }));
    }
    catch (e) {
        throw e;
    }
};
const assertS3Bucket = async (params) => {
    const bucketParams = {
        Bucket: params.bucketName,
    };
    try {
        await params.s3.send(new client_s3_1.CreateBucketCommand(bucketParams));
        (0, utils_log_1.info)('S3 bucket created for storing terraform state', {
            bucketName: bucketParams.Bucket,
        });
    }
    catch (e) {
        // if bucket already exists, ignore error
        if (!(e instanceof client_s3_1.BucketAlreadyOwnedByYou)) {
            (0, utils_log_1.error)('Cannot create bucket ' + params.bucketName + ' error code' + e.code);
            throw new Error('Cannot create S3 state bucket: ' + e.message);
        }
    }
};
const deleteAllObjectsFromBucket = async (s3, bucketName) => {
    let continuationToken = undefined;
    do {
        // List objects in the bucket
        const listResponse = await s3.send(new client_s3_1.ListObjectsV2Command({
            Bucket: bucketName,
            ContinuationToken: continuationToken,
        }));
        // Check if there are any objects to delete
        if (listResponse.Contents && listResponse.Contents.length > 0) {
            // Delete listed objects
            const deleteParams = {
                Bucket: bucketName,
                Delete: {
                    Objects: listResponse.Contents.map((object) => ({ Key: object.Key })),
                },
            };
            await s3.send(new client_s3_1.DeleteObjectsCommand(deleteParams));
        }
        // Check if more objects are to be listed (pagination)
        continuationToken = listResponse.NextContinuationToken;
    } while (continuationToken);
};
const deleteS3Bucket = async (params) => {
    try {
        // First, delete all objects from the bucket
        await deleteAllObjectsFromBucket(params.s3, params.bucketName);
        // Then, delete the empty bucket
        await params.s3.send(new client_s3_1.DeleteBucketCommand({ Bucket: params.bucketName }));
    }
    catch (e) {
        throw e; // Rethrow the error to handle it in the calling code if necessary
    }
};
const deleteState = async (params) => {
    const dynamoDB = new client_dynamodb_1.DynamoDBClient({
        region: params.awsRegion,
        credentials: params.credentials,
    });
    await deleteDynamoDBTable({
        dynamoDB,
        tableName: params.dynamoDBTableName,
    });
    const s3 = new client_s3_1.S3Client({
        region: params.awsRegion,
        credentials: params.credentials,
    });
    await deleteS3Bucket({
        s3,
        bucketName: params.bucketName,
    });
};
exports.deleteState = deleteState;
const assertState = async (params) => {
    (0, utils_log_1.info)('Connecting to Terraform State stored on AWS', {
        bucketName: params.bucketName,
        tableName: params.dynamoDBTableName,
    });
    const dynamoDB = new client_dynamodb_1.DynamoDBClient({
        region: params.awsRegion,
        credentials: params.credentials,
    });
    await assertDynamoDBTable({ dynamoDB, tableName: params.dynamoDBTableName });
    const s3 = new client_s3_1.S3Client({
        region: params.awsRegion,
        credentials: params.credentials,
    });
    await assertS3Bucket({ s3, bucketName: params.bucketName });
    (0, utils_log_1.info)('Connected successfully to Terraform State stored on AWS');
};
exports.assertState = assertState;
//# sourceMappingURL=tfState.js.map